<?php

namespace domain\PointSale\SharedPointSale;

use domain\_\AbstractResolver;
use domain\PointSale\PointSale\PointSale;
use domain\PointSale\PointSale\PointSaleRepository;
use yii\helpers\Html;

class SharedPointSaleResolver extends AbstractResolver
{
    protected PointSaleRepository $pointSaleRepository;
    protected SharedPointSaleRepository $sharedPointSaleRepository;

    public function loadDependencies(): void
    {
        $this->pointSaleRepository = $this->loadService(PointSaleRepository::class);
        $this->sharedPointSaleRepository = $this->loadService(SharedPointSaleRepository::class);
    }

    public function getPointsSaleSharedWithPointSale(PointSale $pointSale, array &$pointsSaleSharedWithPointSaleArray = []): array
    {
        $idProducerContext = $this->getProducerContextId();
        $sharedPointsSaleConfirmedArray = $this->sharedPointSaleRepository->findSharedPointsSaleConfirmedByPointSale($pointSale);

        foreach($sharedPointsSaleConfirmedArray as $sharedPointSaleConfirmed) {
            if ($sharedPointSaleConfirmed->getPointSale()->id != $pointSale->id
                && !in_array($sharedPointSaleConfirmed->getPointSale(), $pointsSaleSharedWithPointSaleArray)
                && $sharedPointSaleConfirmed->getPointSale()->getProducer()->id != $idProducerContext) {

                $pointsSaleSharedWithPointSaleArray[] = $sharedPointSaleConfirmed->getPointSale();
                $this->getPointsSaleSharedWithPointSale($sharedPointSaleConfirmed->getPointSale(), $pointsSaleSharedWithPointSaleArray);
            }
            if ($sharedPointSaleConfirmed->getPointSaleWithSharing()->id != $pointSale->id
                && !in_array($sharedPointSaleConfirmed->getPointSaleWithSharing(), $pointsSaleSharedWithPointSaleArray)
                && $sharedPointSaleConfirmed->getProducerWithSharing()->id != $idProducerContext) {

                $pointsSaleSharedWithPointSaleArray[] = $sharedPointSaleConfirmed->getPointSaleWithSharing();
                $this->getPointsSaleSharedWithPointSale($sharedPointSaleConfirmed->getPointSaleWithSharing(), $pointsSaleSharedWithPointSaleArray);
            }
        }

        return array_unique($pointsSaleSharedWithPointSaleArray, SORT_REGULAR);
    }

    public function getPointsSaleSharedWithPointSaleAsTitle(PointSale $pointSale): string
    {
        $title = '';
        $pointsSaleSharedWithPointSale = $this->getPointsSaleSharedWithPointSale($pointSale);

        foreach($pointsSaleSharedWithPointSale as $pointSaleShared) {
            $title .= Html::encode($pointSaleShared->getProducer()->getName().' - '.$pointSaleShared->getName()).'&#013;';
        }

        return $title;
    }

    public function countPointsSaleSharedWithPointSale(PointSale $pointSale): int
    {
        return count($this->getPointsSaleSharedWithPointSale($pointSale));
    }

    public function hasPointSaleSharedWithPointSale(PointSale $pointSale): bool
    {
        return (bool) $this->countPointsSaleSharedWithPointSale($pointSale);
    }

    public function countPointsSaleShared(): int
    {
        $count = 0;
        $pointsSaleArray = $this->pointSaleRepository->findPointSales();

        foreach($pointsSaleArray as $pointSale) {
            if($this->hasPointSaleSharedWithPointSale($pointSale)) {
                $count ++;
            }
        }

        return $count;
    }

    public function countSharedPointsSaleRequestsOthers(): int
    {
        return count($this->sharedPointSaleRepository->findSharedPointsSaleRequestsOthers());
    }
}